package dev.fyloz.colorrecipesexplorer.config

import dev.fyloz.colorrecipesexplorer.config.properties.CreProperties
import dev.fyloz.colorrecipesexplorer.model.ConfigurationType
import dev.fyloz.colorrecipesexplorer.model.configuration
import dev.fyloz.colorrecipesexplorer.service.create
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import java.io.File
import java.io.FileInputStream
import java.io.FileOutputStream
import java.time.LocalDateTime
import java.util.*

const val CONFIGURATION_FILE_PATH = "config.properties"
const val CONFIGURATION_FILE_COMMENT = "---Color Recipes Explorer configuration---"

@Configuration
class ConfigurationsInitializer(
    private val creProperties: CreProperties
) {
    @Bean
    fun fileConfiguration() = FileConfiguration("${creProperties.configDirectory}/$CONFIGURATION_FILE_PATH")
}

class FileConfiguration(private val configFilePath: String) {
    val properties = Properties().apply {
        with(File(configFilePath)) {
            if (!this.exists()) this.create()
            FileInputStream(this).use {
                this@apply.load(it)
            }
        }
    }

    fun get(type: ConfigurationType) =
        if (properties.containsKey(type.key))
            configuration(
                type,
                properties[type.key] as String,
                LocalDateTime.parse(properties[configurationLastUpdateKey(type.key)] as String)
            )
        else null

    fun set(type: ConfigurationType, content: String) {
        properties[type.key] = content
        properties[configurationLastUpdateKey(type.key)] = LocalDateTime.now().toString()
        save()
    }

    fun save() {
        FileOutputStream(configFilePath).use {
            properties.store(it, CONFIGURATION_FILE_COMMENT)
        }
    }

    private fun configurationLastUpdateKey(key: String) = "$key.last-updated"
}
